//------------------------------------------------------------------------------
// File: ResamplerPlugin.cpp
//------------------------------------------------------------------------------

/*
	based on libresample

	Real-time library interface by Dominic Mazzoni

	Based on resample-1.7:
	http://www-ccrma.stanford.edu/~jos/resample/

	License: LGPL - see the file LICENSE.txt for more information
*/


#include <windows.h>
#include <assert.h>

#include "ResamplerPlugin.h"
#include "include/libresample.h"

#define MAX_CHANNELS 32

class PrivateData
{
public:
	void *res[MAX_CHANNELS];
	float max_peak; // by yesgrey3
	float peak_limit;

	PrivateData(): max_peak(1.0f), peak_limit(1.0f) 
	{
		for( int i=0; i<MAX_CHANNELS; i++ )
		{
			res[i]=0;
		}
	}
	
	~PrivateData()
	{
		for( int i=0; i<MAX_CHANNELS; i++ )
		{
			if( res[i] )
			{
				resample_close(res[i]);
			}
		}
	}
};


#define GETPRIVATEDATA ((PrivateData*)m_privateData)

ResamplerPlugin::ResamplerPlugin()
{ 
	m_privateData = 0;

	SetParameters(2, 1);
	SetRate(44100,44100);
}

ResamplerPlugin::~ResamplerPlugin()
{
	if(m_privateData)
	{
		PrivateData* pdata = GETPRIVATEDATA;
		delete pdata;
	}
}


bool ResamplerPlugin::SetParameters(int quality, int channelCount, float peak_limit )
{
	if(m_privateData)
	{
		PrivateData* pdata = GETPRIVATEDATA;
		delete pdata;
		m_privateData = 0;
	}

	if( channelCount > MAX_CHANNELS || 0 == channelCount )
	{
		return false;
	}

	m_privateData = new PrivateData;

	GETPRIVATEDATA->max_peak = peak_limit; 
	GETPRIVATEDATA->peak_limit = peak_limit; 

	m_channelCount = channelCount;

	for( int i=0; i<channelCount; i++ )
	{
		/* quality 2 is "good" in ReClock config. "Very good" and "excellent" will
		   enable high quality resampling
	    */

		GETPRIVATEDATA->res[i] = resample_open( quality > 2 ? 1 : 0, 1.0 / 12.0, 12.0 );
		if( 0 == GETPRIVATEDATA->res[i] )
		{
			return false;
		}
	}

	return true;
}

bool ResamplerPlugin::SetRate(int inRate, int outRate)
{
	m_inRate = inRate;
	m_outRate = outRate;
	return true;
}

void ResamplerPlugin::ProcessFlow(float *inBuffer, int inSamples, float *outBuffer, int *outSamples, int maxOutSamples)
{
	*outSamples = 0;

	if (m_channelCount && m_inRate && m_outRate && GETPRIVATEDATA )
	{
		int input_frames = inSamples / m_channelCount;
		int output_frames = maxOutSamples / m_channelCount;
		float *data_in = inBuffer;
		float *data_out = outBuffer;
		double src_ratio = (double)m_outRate/(double)m_inRate;
		int output_frames_gen = 0;

		for( int channel=0; channel<m_channelCount; channel++ )
		{
			int inBufferUsed;
			output_frames_gen += resample_process(GETPRIVATEDATA->res[channel],
                     src_ratio,
                     data_in,
                     input_frames,
                     false,
                     &inBufferUsed,
                     data_out,
                     output_frames,
					 channel,
					 m_channelCount);
		}

		*outSamples = output_frames_gen;
	}
// by yesgrey3 - start
// if samples > max_peak, normalize using peak_limit/max_peak to avoid clipping
   for(int o = 0; o < *outSamples; ++o)
   {
      if (outBuffer[o] > GETPRIVATEDATA->max_peak)
	  {
		  GETPRIVATEDATA->max_peak = outBuffer[o];
	  }
   }
   if (GETPRIVATEDATA->max_peak > GETPRIVATEDATA->peak_limit)
   {
      float xnorm = GETPRIVATEDATA->peak_limit/GETPRIVATEDATA->max_peak;
      for(int o = 0; o < *outSamples; ++o)
	  {
         outBuffer[o] = outBuffer[o]*xnorm;
	  }
   }
// by yesgrey3 - end
}

void ResamplerPlugin::Flush()
{
//	Sleep( 1000 );
}
